#include "CyanStripe.hpp"

// C++ system level
#include <cstdio>     // sprintf
// #include <functional>

// ESP32 specific
#include "esp_log.h"
#include "driver/gpio.h"

// project specific
#include <Types.hpp>
#include "SiliconTorch/NVSExplorer.hpp"

// qthing stuff
#include <qthing>
#include "SiliconTorch/FxCyanRGB8.hpp"




// TODO: implement!







  /* auto led0 = cfg.add<qthing::leds::WS2812B_V3>(GPIO_NUM_27, 8);

  cyanRGB = new SiliconTorch::CyanStripe::CyanStripe(8);
  cyanRGB->registerUDPHandler();
  cyanRGB->registerAtCyanBus(*cyanBus);

  auto bullshitPtr = std::shared_ptr<qthing::Animation<qthing::RGB>>(cyanRGB, [](void*){});
  cfg.add<qthing::Animator<qthing::RGBW8>>(led0)->setAnimation(bullshitPtr); */




namespace SiliconTorch {

  namespace Service {

    namespace CyanStripe {

      void CyanStripe::init() {
        setIcon("🎨");
        setName("CyanStripe");
        setNameSpace("CyanStripe");

        addWantedService("CyanBus");
      }

      void CyanStripe::start() {

        for (u8 idx = 0; idx < MAX_CHANNELS; idx++) {

          ChannelInfo info = readChannelData(idx);

          if (!info.valid) break;

          ESP_LOGI(getName().c_str(), "Configuring: FxCyanRGB8{ info[ %d ] gpio[ %d ] leds[ %d ] startIdx[ %d ] colorOrder[ %s ] }",
            idx,
            info.gpio,
            info.leds,
            info.startIdx,
            FxCyanRGB8::colorOrder2String(info.colorOrder).c_str()
          );

          ChannelDriver channel = {
            .info    = info,
            .cyanRGB = new SiliconTorch::FxCyanRGB8::FxCyanRGB8(info.leds, info.startIdx)
          };

          channels.push_back(channel);

          channel.cyanRGB->registerUDPHandler();

          // TODO: Get CyanBus from ServiceManager if it exists
          // channel.cyanRGB->registerAtCyanBus(*cyanBus);
        }

      }


      void CyanStripe::configureQthingLEDs(qthing::Config* cfg) const {

        for (const auto& channel : channels) {

          // TODO: Sadly this is a block of *ughh* which we can't write any better :/

          auto leds = cfg->add<qthing::leds::WS2812B_V3>((gpio_num_t)(channel.info.gpio), channel.info.leds);
          auto bullshitPtr = std::shared_ptr<qthing::Animation<qthing::RGB>>(channel.cyanRGB, [](void*){});
          cfg->add<qthing::Animator<qthing::RGBW8>>(leds)->setAnimation(bullshitPtr);
        }
      }


      ChannelInfo CyanStripe::readChannelData(u8 channel) const {

        NVSExplorer::NVSExplorer& nvs = NVSExplorer::NVSExplorer::instance();

        ChannelInfo ch;

        char tmp[32];

        std::snprintf(tmp, sizeof(tmp), "ch%d_gpio", channel);
        ch.gpio = nvs.getUnsignedInt(getNameSpace(), std::string(tmp), 0xFF);

        std::snprintf(tmp, sizeof(tmp), "ch%d_leds", channel);
        ch.leds = nvs.getUnsignedInt(getNameSpace(), std::string(tmp), 0);

        std::snprintf(tmp, sizeof(tmp), "ch%d_startIdx", channel);
        ch.startIdx = nvs.getUnsignedInt(getNameSpace(), std::string(tmp), 0);

        std::snprintf(tmp, sizeof(tmp), "ch%d_colorOrder", channel);
        str _colorOrder = nvs.getString(getNameSpace(), std::string(tmp), "RGB");
        ch.colorOrder   = FxCyanRGB8::string2ColorOrder(_colorOrder);

        ch.valid = ch.gpio < 0xFF && ch.leds > 0;

        return ch;
      }


    }
  }
}


