#include "LambdaTask.hpp"

// ESP32 system level
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"

// project specific
#include <SpiderLib/NumberTypes.hpp>



static void taskWrapper(void* body) {
  std::function<void()>* f = (std::function<void()> *)body;
  (*f)();
}

static u16 taskID = 0;
static u16 nextID() {
  return taskID++;  // TODO: increment thread-safe!
}

SpiderLib::Util::LambdaTask::LambdaTask(SpiderLib::Util::Function0 body, u32 stackSize) : body(body) {

  char taskName[32];
  sprintf(taskName, "LambdaTask[%d]", nextID());

  // TODO: allow other niceness than one
  // TODO: allow pinning to other cores than app CPU

  u8 priority = 1;

  std::function<void()> _body = [&]() {
    this->body();
    this->_running = false;  // TODO: thread safe??? needed?
    vTaskDelete(NULL);
  };

  std::function<void()>* __body = new std::function<void()>(_body);

  xTaskCreatePinnedToCore(taskWrapper, taskName, stackSize, (void*)__body, priority, &taskHandle, 1);

}

TaskHandle_t SpiderLib::Util::LambdaTask::getHandle() {
  return taskHandle;
}

bool SpiderLib::Util::LambdaTask::running() {
  return _running;
}

// TODO: thread safe??? needed?
void SpiderLib::Util::LambdaTask::kill() {
  if (_running) vTaskDelete(getHandle());
  _running = false;
}
